<?php
/**
 * @package GeekXpressDelivery
 * @version 2.0
 */
/*
/**
 * Plugin Name: Delivery Management For Woocommerce
 * Description: This plugin connects your woocommerce store to XpressDelivery App - Delivery Management System and then sends all orders from your woocommmerce store to XpressDelivery as tasks to be auto assigned to your drivers for delivery. Drivers also have their own android & iOS apps.
 * Author: xpressdelivery.app
 * Author URI: https://xpressdelivery.app/
 * Version: 2.0
 */

require_once plugin_dir_path( __FILE__ ) . 'include/activate.php';
require_once plugin_dir_path( __FILE__ ) . 'include/deactivate.php';
require_once plugin_dir_path( __FILE__ ) . 'include/xpressdelivery-api.php';

register_activation_hook( __FILE__, array('GeekSerialNumberActivate','activate') );
register_deactivation_hook( __FILE__, array('GeekSerialNumberDeactivate','deactivate') );

class GeekXpressDelivery {

	function check_requirements() {        
	    if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		    $this->register();
		}else{
			add_action( 'admin_notices', function () {
                echo '<div class="notice notice-error is-dismissible"><p>' . __( '<strong>WooCommerce connect to Xpress Delivery</strong> requires active WooCommerce!',
                        'woocommrce-xpress-delivery' ) . '</p></div>';
                return;
            } );
		}
    }

	function register(){

		add_action( 'admin_enqueue_scripts', array($this,'backendEnqueue'));
		add_action('admin_menu', array($this,'register_my_custom_submenu_page'));

		$type = get_option('xpress_status_type');

		if (in_array("wc-pending",$type)) {
			add_action('woocommerce_order_status_pending', array($this,'order_pending'));
			 
		}
		if (in_array("wc-processing",$type)) {
		  	add_action('woocommerce_order_status_processing', array($this,'order_processing'));

		}
		if (in_array("wc-on-hold",$type)) {
			add_action('woocommerce_order_status_on-hold', array($this,'order_on_hold'));
		}
		if (in_array("wc-completed",$type)) {
			add_action('woocommerce_order_status_completed', array($this,'order_complete'));
		}
		if (in_array("wc-cancelled",$type)) {
			add_action('woocommerce_order_status_cancelled', array($this,'order_cancel'));
		}
	}

	function order_cancel($order_id){
	  $sent_order = $this->check_order($order_id,true);
		if($sent_order==1){
			$this->sendData($order_id,true);
		}
	}
	
	function order_complete($order_id){
	$sent_order = $this->check_order($order_id,true);
		if($sent_order==1){
			$this->sendData($order_id,true);
		}
	}
	
	function order_on_hold($order_id){
		$sent_order = $this->check_order($order_id,true);
		if($sent_order==1){
			$this->sendData($order_id,true);
		}
	}
	
	function order_processing($order_id){
	$sent_order = $this->check_order($order_id,true);
		if($sent_order==1){
			$this->sendData($order_id,true);
		}
	}
	
	function order_pending($order_id){
	$sent_order = $this->check_order($order_id,true);
		if($sent_order==1){
			$this->sendData($order_id,true);
		}
	}
	
	function check_order($order_id){
		$order = wc_get_order( $order_id );
		$items = $order->get_items();  
		$total_items = count($items);
		$product_type = get_option('xpress_product_type');
		if($product_type == "true"){
			return 1;
		}else{
			foreach ( $items as $key => $item ) {
				$product = wc_get_product($item['product_id'] );
				if($product->is_downloadable() || $product->is_virtual()){
					$res1[$key]['type'] = $product->is_downloadable() || $product->is_virtual();
				}
			}

			if($total_items > count($res1)){
				return 1;
			}else{
				return 0;
			}
		}	
	}

	function sendData($order_id,$note_show = false){
		$key   = get_option('xpress_key');
		$token = get_option('xpress_merchant_token');
		$trans = get_option('xpress_trans_type');
		$minut = ((int)get_option('xpress_delivery_hours'))*60;
		$order = wc_get_order( $order_id );
		$ship  = $order->needs_shipping_address();
		$date  = date("Y-m-d H:i:s",strtotime('+'.$minut.' minutes',strtotime($order->get_date_created())));
// 	echo $date;
// 	die('nnnn');
		$email = $order->get_billing_email();
		$note  = $order->get_customer_note();
		$contact_number = $order->get_billing_phone();
// 		echo $contact_number;
// 		die();


		if ($note_show) {
			$note  = urlencode($this->order_html($order));
		}

		

		// echo "<br>".$order->get_date_created(); die();
		if ($ship) {
			$name  		  = $order->get_shipping_first_name()." ".$order->get_shipping_last_name();
			$delivery_add = wp_kses_post( $order->get_formatted_shipping_address( __( 'N/A', 'woocommerce' ) ) );
		}else{
			$name  		  = $order->get_billing_first_name()." ".$order->get_billing_last_name();
			$delivery_add = wp_kses_post( $order->get_formatted_billing_address( __( 'N/A', 'woocommerce' ) ) );
		}

//code by kalim start
		//$key='16b04f3bac9aa2909ac1787f4eb25dd8';
		$note="OrderNO-".$order_id." Name-".$name." Date-".$date." Address-".$delivery_add;
//code by kalim end		

		$url = "https://xpressdelivery.app/db/api_services/insert_task?keys=$key&merchant_token=$token&order_id=$order_id&contact_number=$contact_number&task_description=$note&trans_type=$trans&email_address=$email&customer_name=$name&delivery_date=$date&delivery_address=$delivery_add";
		
		// echo '<pre>';
		// print_r($key);
		// print_r($url);
		// die('nnn');
		return wp_remote_post($url);

	}

	function backendEnqueue(){
		wp_enqueue_style( 'xpressdelivery', plugins_url( '/assets/css/style.css', __FILE__ ), array(), false, false);
		wp_enqueue_style( 'chosen', plugins_url( '/assets/css/chosen.min.css', __FILE__ ), array(), false, false);
		wp_enqueue_script( 'chosen', plugins_url( '/assets/js/chosen.jquery.min.js', __FILE__ ), array(), false, true);

	}
	function register_my_custom_submenu_page() {
	    add_submenu_page( 'woocommerce', 'XpressDelivery App', 'XpressDelivery App', 'manage_options', 'woocommrce-xpress-delivery', array($this,'my_plugin_setting_page') ); 
	    // update_option
	}
	function my_plugin_setting_page() {
		$nonce = (!empty($_POST['nonce'])) ? $_POST['nonce'] : '' ;	
		if (isset($_POST['xpress_form']) && wp_verify_nonce( $nonce, 'form-nonce')){
			unset($_POST['xpress_form']);
			
			if(!array_key_exists("xpress_product_type",$_POST)){
					$_POST['xpress_product_type']='false';
				}
			if(!array_key_exists("synchonize_status",$_POST)){
					$_POST['synchonize_status']='false';
				}

			foreach ($_POST as $key => $value) {
				if ($key == "xpress_delivery_hours") {
					if (!preg_match('/^[1-9][0-9]*$/',$value)) {
						$value = 0;
					}
				}
				update_option($key,$value);
			}
		}
		//code by kalim to sanitize start
		// $wooStatus = array();
		// if(!empty(get_option('xpress_status_type'))){
		// 	$wooStatus = get_option('xpress_status_type');			
		// }
		$wooStatus = array();
		if(!empty(get_option('xpress_status_type'))){
			$wooStatus = sanitize_text_field(get_option('xpress_status_type'));			
		}
		//code by kalim to sanitize end 
		?>
		<?php $orderStatus = wc_get_order_statuses(); ?>
		<div class="wrap">
    		<h1>XpressDelivery App Settings</h1>
    		<form class="form" action="" method="post" id="xpress-form">
    			<input type="hidden" name="nonce" value="<?php echo wp_create_nonce('form-nonce'); ?>">
    			<table class="form-table">
    				<tr valign="top" style="display:none;">
				        <th scope="row">Keys</th>
				       <td><input required="" type="text" name="xpress_key" class="api_key" value="<?php echo esc_attr('16b04f3bac9aa2909ac1787f4eb25dd8'); ?>" /></td>
				    </tr>
				    <tr valign="top">
				        <th scope="row">Merchant Token</th>
				        <td><input type="text" required="" name="xpress_merchant_token" class="merchant_token" value="<?php echo esc_attr( get_option('xpress_merchant_token') ); ?>" /></td>
				    </tr>
				    <tr valign="top">
				        <th scope="row">Woocommerce Status</th>
				        <td>
				        	<select name="xpress_status_type[]" class="chosen-select" multiple="multiple">
				        	<?php foreach ($orderStatus as $key => $value) {
								//code by kalim to sanitize start  
				        		$key = esc_attr($key);
				        		$value = esc_attr($value); 
				        	//code by kalim to sanitize end 
							 ?>
								
				        		<?php if ($key == 'wc-refunded' || $key == 'wc-failed') {
				        			continue;
				        		} ?>
				        		<option value="<?php echo $key; ?>" <?php echo (in_array($key,$wooStatus) ? "selected" : "") ?>><?php echo $value; ?></option>
				        		<!-- <input type="text" name="xpress_status_type" value="<?php echo esc_attr( get_option('xpress_status_type') ); ?>" /></td> -->
				        	<?php } ?>
				        </td>
				    </tr>
				    <tr valign="top">
				        <th scope="row">Transaction Type</th>
				        <td>Delivery &nbsp;<input type="radio" name="xpress_trans_type" <?php echo (get_option('xpress_trans_type') == "delivery" ? "checked" : "checked") ?> value="delivery" /> &nbsp;&nbsp;&nbsp;Pickup &nbsp;<input type="radio" name="xpress_trans_type" <?php echo (get_option('xpress_trans_type') == "pickup" ? "checked" : "") ?> value="pickup" /></td>
				    </tr>
				    <tr valign="top">
							<th scope="row">Product Type</th>
							<td>Virtual or Downloadable &nbsp;<input type="checkbox" name="xpress_product_type" <?php echo (get_option('xpress_product_type') == "true" ? "checked" : " ") ?> value="true" /></td>
							
						</tr>
						<tr>
							<td></td>
							<td style="font-size: 12px;">Check to send the order with virtual or downloadable product to xpress delivery.</td>
						</tr>
						<tr valign="top">
							<th scope="row">Order Status</th>
							<td>Synchronize with xpressdelivery status &nbsp;<input type="checkbox" name="synchonize_status" <?php echo (get_option('synchonize_status') == "true" ? "checked" : " ") ?> value="true" /></td>
							
						</tr>
						<tr>
							<td></td>
							<td style="font-size: 12px;">Check to synchronize woocommerce status with xpressdelivery order status.</td>
						</tr>
						
				    <tr valign="top">
				        <th scope="row">Delivery Hours</th>
				        <td><input type="number" name="xpress_delivery_hours" value="<?php echo esc_attr( get_option('xpress_delivery_hours') ); ?>" /></td>
				    </tr>
				    <tr>
				    	<td colspan="2"><strong>Note:- </strong>Plz enter hours only in multiples of 24 hours.</td>
				    </tr>
				    <tr>
							<td colspan="2"><strong>Important Note:- </strong>After updating all details and saving in the plugin plz go to your account XpressDelivery acc. settings here https://xpressdelivery.app/db/app/settings and update and save your woocommerce URL here. If this is not done then integration will not work.</td>
						</tr>
				    <tr valign="top">
				        <td colspan="2"><input type="submit" name="xpress_form" value="submit"></td>
				    </tr>
    			</table>
    		</form>
    	</div>
    	<script type="text/javascript">
    		jQuery(document).ready(function($){
    			$(".chosen-select").chosen({
				  no_results_text: "Oops, nothing found!"
				});
				$("form#xpress-form").submit(function(){
					// alert($("input[name=xpress_delivery_hours]").val()%24);
					if ($("input[name=xpress_delivery_hours]").val()%24 != 0 && ($("input[name=xpress_delivery_hours]").val() != 0 || $("input[name=xpress_delivery_hours]").val() != "")) {
						alert('Plz enter hours only in multiples of 24 hours.');
						return false;
					}
				})
    		});
		</script>
	<?php }
	function order_html($order){
		$order_items     	= $order->get_items( apply_filters( 'woocommerce_purchase_order_item_types', 'line_item' ) );
		$orderHtml = '<div class="woocommerce-order"><ul class="woocommerce-order-overview woocommerce-thankyou-order-details order_details">
			<li class="woocommerce-order-overview__order order">
				Order number:
				<strong>'.$order->get_order_number().'</strong>
			</li>
			<li class="woocommerce-order-overview__date date">
				Date:
				<strong>'.wc_format_datetime( $order->get_date_created() ).'</strong>
			</li>
			<li class="woocommerce-order-overview__total total">
				Total:
				<strong>'.$order->get_formatted_order_total().'</strong>
			</li>';

		if ( $order->get_payment_method_title() ) : 
			$orderHtml .= '<li class="woocommerce-order-overview__payment-method method">
				Payment method:
				<strong>'.wp_kses_post( $order->get_payment_method_title() ).'</strong>
			</li>';
		endif;
		$orderHtml .= '</ul>
				<section class="woocommerce-order-details">
					<h2 class="woocommerce-order-details__title">Order details</h2>

					<table class="woocommerce-table woocommerce-table--order-details shop_table order_details">

						<thead>
							<tr>
								<th class="woocommerce-table__product-name product-name">Product</th>
								<th class="woocommerce-table__product-table product-total">Total</th>
							</tr>
						</thead>
						<tbody>';
		foreach ( $order_items as $item_id => $item ) {
			$product = $item->get_product();
			$is_visible        = $product && $product->is_visible();
			$product_permalink = apply_filters( 'woocommerce_order_item_permalink', $is_visible ? $product->get_permalink( $item ) : '', $item, $order );
			$orderHtml .= '<tr>
				<td class="woocommerce-table__product-name product-name">
					'.apply_filters( 'woocommerce_order_item_name', $product_permalink ? sprintf( '<a href="%s">%s</a>', $product_permalink, $item->get_name() ) : $item->get_name(), $item, $is_visible ).''.
					apply_filters( 'woocommerce_order_item_quantity_html', ' <strong class="product-quantity">' . sprintf( '&times; %s', $item->get_quantity() ) . '</strong>', $item ).
				'</td>
				<td class="woocommerce-table__product-total product-total">
					'.$order->get_formatted_line_subtotal( $item ).'
				</td>
			</tr>';
		}
		$orderHtml .= '</tbody>
					<tfoot>';
		foreach ( $order->get_order_item_totals() as $key => $total ) {
			$orderHtml .= '<tr>
				<th scope="row">'.$total['label'].'</th>
				<td>'.$total['value'].'</td>
			</tr>';
		}
		if ( $order->get_customer_note() ) :
			$orderHtml .='<tr>
							<th>Note:</th>
							<td>'.wptexturize( $order->get_customer_note() ).'</td>
						</tr>';
		endif; 
		$orderHtml .='</tfoot>
			</table>';
		return $orderHtml;
	}
}

$xpressdelivery = new GeekXpressDelivery();
$xpressdelivery->check_requirements();